function box=findBB(sil)
% box=findBB(sil)
% 
% return coordinates in the image of the square boundig box centered in
% the median of the silhouette
% fields of box:
% -  size
% -  tlRow : row of the top left corner
% -  tlCol : column ======"====== 
% Note: negative coord. are possible, and the BB may extend out of the
% image (in which case it should be padded with zeros).
% -  startRow : first row of the in-image part
% -  endRow : last row of the in-image part
% -  padRowT,padRowB : number of rows to pad top/bottom
% -  startCol, endCol : similarly for columns
% -  padColL,padColR : left/right padding
% -  imRows,imCols : number of actual image rows/columns extracted

% find FG pixels
[i,j]=find(sil);
% and their medians
medRow=round(median(i));
medCol=round(median(j));

% compute extent around the median
spreadRow=max(medRow-min(i),max(i)-medRow);
spreadCol=max(medCol-min(j),max(j)-medCol);
sz=max(spreadRow,spreadCol);

% calculate size (maximum of the width and height of the rectangle)
box.size=2*sz;
% top left rox and col
box.tlRow=medRow-sz;
box.tlCol=medCol-sz;

% corners within the image:
% startRow: first row WITHIN THE IMAGE
box.startRow = max(1,box.tlRow);

% padRowT: number of rows to be added on top of rectangle
box.padRowT = box.startRow-box.tlRow;

% same for first column and left padding
box.startCol = max(1,box.tlCol);
box.padColL = box.startCol-box.tlCol;

box.endRow = min(size(sil,1),box.tlRow+box.size-1);
box.endCol = min(size(sil,2),box.tlCol+box.size-1);

% number of rows and columns to be added as padding
box.padColR = box.tlCol+box.size-1-box.endCol;
box.padRowB = box.tlRow+box.size-1-box.endRow;

% num. of rows and columns FROM IMAGE used in the BB
box.imRows = box.endRow - box.startRow +1;
box.imCols = box.endCol - box.startCol +1;
