function [x,edges,maps]=readed(im,blocks,t,meth,sil,addcont)
%  x=readed(im,blocks,t,meth)
%
%  Constructs edge direction histogram from IM, using subwindows defined by
%  BLOCKS (created by buildbox). Edges are detected using method METH
%  (default 'sobel'), with threshold T (default .05)
% 
%  x=readed(im,blocks,t,meth,cont)
%  adds to the edge map the contour cont.
%  
%  The bins are ordered by maps (all the boxes for first orientation,
%  then second orientation etc.)
% 
% [x,edges] = readed(...) returns also the binary edge image
% [x,edges,maps]=readed(...) returns in maps a cell array of edge maps
% for each orientation.

l = size(blocks,2);
if (nargin < 3)
  t = .05;
end

if (nargin < 4)
  meth='sobel';
end

if (nargin < 5)
  sil=[];
end

[edges,maps]=edgedir(im,meth,t,sil,addcont);


x=uint8(zeros(1,l*length(maps)));
for i=1:length(maps)
  ii = iimage(maps{i});
  x((i-1)*l+1:i*l)=reshape(boxf(ii,blocks),1,[]);
end
