# Methods for detailed manipulation of Poser hand models

# (C) Greg Shakhnarovich, TTI-Chicago, 2008

import poser
import random
import sys, time, os

# Auxiliary stuff

unirnd = random.uniform

__finger_names = ['Index','Mid','Ring','Pinky']

#__cam_names = map(lambda x: x+' Camera', ['Front','Main','Aux','Top','Left','Bottom'])
__cam_names = map(lambda x: x+' Camera', ['Dolly','Main','Aux'])

scene = poser.Scene()


def all_set_lim(which,actor,valm,ValM):
    actor.Parameter(which).SetMinValue(valm);
    actor.Parameter(which).SetMaxValue(ValM);
    for c in actor.Children():
        all_set_lim(which,c,valm,ValM)

def all_set_scale(which,actor,val):
    # recursively set the corresp. scale (x,y,z) to actor and its descendants
    actor.SetParameter(which+'Scale',val)
    for c in actor.Children():
        all_set_scale(which,c,val)


def phalangs(name):
    return [
        scene.Figure("Figure 1").Actor(name + '1'),
        scene.Figure("Figure 1").Actor(name + '2'),
        scene.Figure("Figure 1").Actor(name + '3')]


def draw_and_save(fname,fmt,logf,rend=1):
    scene.SetRenderAntiAliased(1)
    scene.SetRenderToNewWindow(0)
    logged = 0;
    for cam in __cam_names:
        let = cam[0]
        scene.SetCurrentCamera(scene.Actor(cam))
        if rend:
            scene.Render()
        else:
            scene.DrawAll()
        scene.AntialiasNow()
        scene.SaveImage(fmt,fname+'_'+let)
        vals={}; vals['file'] = fname+'_'+let+'.'+fmt
        if not logged:
            hp = poserio.HandParam(0,vals)
            hp.write(logf)
            logged = 1
    logged = 0

# functions for hand perturbation

def bend_finger(finger, bnd=None):
    # finger is given by name: 'rIndex | rMid | rRing | rPinky'
    # val is in persentage of bending - positive or negative...

    ph = phalangs(finger);
    # default - a random value
    if bnd == None:
        bnd = unirnd(-1,1)
    # find the limits of bend
    if bnd > 0:
        lim = map(lambda x: x.Parameter('Bend').MaxValue(), ph)
    else:
        lim = map(lambda x: x.Parameter('Bend').MinValue(), ph)

    #compute actual bend values
    vals = map(lambda x,y: x*y, [abs(bnd)]*len(lim), lim)
    # compute the needed deformation
    map(lambda x,y: x.Parameter('Bend').SetValue(y), ph,vals)


def grasp(side,val):
    # bend all the fingers together
    bend_finger(side+'Index',val)
    bend_finger(side+'Mid',val)
    bend_finger(side+'Ring',val)
    bend_finger(side+'Pinky',val)

def forearm():
    fa = scene.Actor('Right Forearm')
    for parmname in ['Bend','Twist','Side-Side']:
        parm = fa.Parameter(parmname)
        parm.SetValue(unirnd(parm.MinValue(),parm.MaxValue()))
    
    

def spread(side,val = None):
    #val is percentage 0..1
    if val == None:
        val = unirnd(-1,1)
    ss = map(lambda x: scene.Actor(side+ ' '+x+' 1').Parameter('Side-Side'),
        __finger_names)
    # index and mid: positive side-side
    ss[0].SetValue(ss[0].MaxValue()*val)
    ss[1].SetValue(ss[1].MaxValue()*val)
    # ring, pinky - negative...
    ss[2].SetValue(ss[2].MinValue()*val)
    ss[3].SetValue(ss[3].MinValue()*val)
    

